Crate cdx[−][src]
The command line tool cdx is a set of tools for text file manipulation
and command line data mining.
It is hoped that the associated library will be useful for theid party tools.
Modules
Test if a line matches some criterea
Handles conversion between named column sets and lists of column numbers Also helps with selecting those columns from a line of text
Tools for comparing lines and fields
Floating Point expressions
Join files together by matching column values
Tools for sorting text files
The Text trait, with implementations for str and &u8 for all those things that are needed for both types
parse and run tooltest files
Macros
Structs
pointers into a vector, simulating a slice without the ownership issues
Object to enforce HeaderMode
Input file. Wrapped in a type so I can ‘impl Debug’
shared context for any input file type
File reader for text file broken into lines with columns previous N lines are stil available
File reader for text file broken into lines with columns
as TextLine, but String rather than Vec
Iterator over the columns in a StringLine
A line of a text file, broken into fields.
Access to the lines and parts is allowed, but should seldom be necessary
line includes the trailing newline, but no field contains the newline
An empty line contains one empty field
Iterator over the columns in a TextLine
Enums
Various errors
How to combine headers from multiple sources
Constants
strings associated with HeaderMode
Functions
copy r to w
Make an Infile from a file name
Make an Infile from a file name
Make an Outfile from a file name
Make an Outfile from a file name
Type Definitions
Result type for cdx